"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const ast_utils_1 = require("../utils/ast-utils");
exports.default = (0, utils_1.createRule)("block-lang", {
    meta: {
        docs: {
            description: "disallows the use of languages other than those specified in the configuration for the lang attribute of `<script>` and `<style>` blocks.",
            category: "Best Practices",
            recommended: false,
        },
        schema: [
            {
                type: "object",
                properties: {
                    enforceScriptPresent: {
                        type: "boolean",
                    },
                    enforceStylePresent: {
                        type: "boolean",
                    },
                    script: {
                        oneOf: [
                            {
                                type: ["string", "null"],
                            },
                            {
                                type: "array",
                                items: {
                                    type: ["string", "null"],
                                },
                                minItems: 1,
                            },
                        ],
                    },
                    style: {
                        oneOf: [
                            {
                                type: ["string", "null"],
                            },
                            {
                                type: "array",
                                items: {
                                    type: ["string", "null"],
                                },
                                minItems: 1,
                            },
                        ],
                    },
                },
                additionalProperties: false,
            },
        ],
        messages: {},
        type: "suggestion",
    },
    create(context) {
        const enforceScriptPresent = context.options[0]?.enforceScriptPresent ?? false;
        const enforceStylePresent = context.options[0]?.enforceStylePresent ?? false;
        const scriptOption = context.options[0]?.script ?? null;
        const allowedScriptLangs = Array.isArray(scriptOption)
            ? scriptOption
            : [scriptOption];
        let scriptLang = null;
        let scriptNode = undefined;
        const styleOption = context.options[0]?.style ?? null;
        const allowedStyleLangs = Array.isArray(styleOption)
            ? styleOption
            : [styleOption];
        let styleLang = null;
        let styleNode = undefined;
        return {
            SvelteScriptElement(node) {
                scriptNode = node;
                scriptLang = (0, ast_utils_1.getLangValue)(node)?.toLowerCase() ?? null;
            },
            SvelteStyleElement(node) {
                styleNode = node;
                styleLang = (0, ast_utils_1.getLangValue)(node)?.toLowerCase() ?? null;
            },
            "Program:exit"() {
                if (!allowedScriptLangs.includes(scriptLang)) {
                    if (scriptNode !== undefined) {
                        context.report({
                            node: scriptNode,
                            message: `The lang attribute of the <script> block should be ${prettyPrintLangs(allowedScriptLangs)}.`,
                        });
                    }
                }
                if (scriptNode === undefined && enforceScriptPresent) {
                    context.report({
                        loc: { line: 1, column: 1 },
                        message: `The <script> block should be present and its lang attribute should be ${prettyPrintLangs(allowedScriptLangs)}.`,
                    });
                }
                if (!allowedStyleLangs.includes(styleLang)) {
                    if (styleNode !== undefined) {
                        context.report({
                            node: styleNode,
                            message: `The lang attribute of the <style> block should be ${prettyPrintLangs(allowedStyleLangs)}.`,
                        });
                    }
                }
                if (styleNode === undefined && enforceStylePresent) {
                    context.report({
                        loc: { line: 1, column: 1 },
                        message: `The <style> block should be present and its lang attribute should be ${prettyPrintLangs(allowedStyleLangs)}.`,
                    });
                }
            },
        };
    },
});
function prettyPrintLangs(langs) {
    const hasNull = langs.includes(null);
    const nonNullLangs = langs
        .filter((lang) => lang !== null)
        .map((lang) => `"${lang}"`);
    if (nonNullLangs.length === 0) {
        return "omitted";
    }
    const hasNullText = hasNull ? "either omitted or " : "";
    const nonNullText = nonNullLangs.length === 1
        ? nonNullLangs[0]
        : `one of ${nonNullLangs.join(", ")}`;
    return hasNullText + nonNullText;
}
